/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt Charts module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:GPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 or (at your option) any later version
** approved by the KDE Free Qt Foundation. The licenses are as published by
** the Free Software Foundation and appearing in the file LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "penwidget.h"
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QSpinBox>

PenWidget::PenWidget(QWidget *parent):QWidget(parent),
m_colorComboBox(0),
m_lineStyleComboBox(0),
m_widthSpinBox(0)
{
    createContent();
    createLayout();
    setFixedSize(minimumSizeHint());
}

PenWidget::~PenWidget()
{

}

void PenWidget::createContent()
{
    m_colorComboBox = new QComboBox(this);
    m_lineStyleComboBox = new QComboBox(this);
    m_widthSpinBox = new QSpinBox(this);
}

void PenWidget::createLayout()
{
    QFormLayout *layout = new QFormLayout();
    layout->addRow(tr("Color"), m_colorComboBox);
    layout->addRow(tr("Style"), m_lineStyleComboBox);
    layout->addRow(tr("Width"), m_widthSpinBox);
    setLayout(layout);
}

